angular
  .module("starter.controllers")

  // 图书详情
  .controller(
    "BookDetailCtrl",
    function (
      $controller,
      $rootScope,
      $scope,
      $stateParams,
      $ionicHistory,
      $state,
      $ionicLoading,
      $ionicPopup,
      apiService,
      bookService,
      ngCordovaService,
      uiService,
      $ionicTabsDelegate,
      broadcastConstants,
      broadcastService
    ) {
      if (!$scope.user) {
        $state.go("tab.mine", { fromState: $ionicHistory.currentStateName() });
        return;
      }
      // ----------------------------------------------------------------------------------------------------------------
      $scope.book = $stateParams.book; // 当前图书
      console.log(
        "BookDetailCtrl. user is " + angular.toJson($rootScope.user, true)
      );
      $scope.allowLoanedBooks = []; // 可借阅图书
      $scope.totalBooks = []; // 所有图书
      $scope.comments = []; // 图书评论
      $scope.nextAllowLoanedBookIndex = 0; // 尝试借阅可借图书的当前索引号
      $scope.loanedStatus = "";
      $scope.bookDetail = {
        accountId: $rootScope.user.account,
        borrowerId: $rootScope.user.borrowID,
        borrowerName: $rootScope.user.name,
        isbn: $scope.book.isbn,
        groupBookId: $scope.book.groupBookID,
        bookTitle: $scope.book.title,
        bookAuthor: $scope.book.author,
        bookImage: $scope.book.image.medium,
        bookCount: 1,
      };
      $scope.needBook = {
        accountId: $rootScope.user.account,
        borrowerId: $rootScope.user.borrowID,
        borrowerName: $rootScope.user.name,
        isbn: $scope.book.isbn,
        bookGroupId: $scope.book.groupBookID,
        bookTitle: $scope.book.title,
        bookAuthor: $scope.book.author,
        bookImage: $scope.book.image.medium,
      };
      // ----------------------------------------------------------------------------------------------------------------
      $scope.retrieveBookDetail = function () {
        uiService.ionicLoadingShow({ templateTitle: "加载中..." });
        apiService.retrieveBookDetail(
          $scope.book.groupBookID,
          $rootScope.userInfo.account,
          callback
        );
      };
      // 获取图书详情的回调方法
      var callback = {
        onSuccess: onSuccess,
        onError: onError,
        onFinally: onFinally,
      };
      function onSuccess(data, status, config, headers) {
        if (data) {
          $scope.allowLoanedBooks = [];
          angular.forEach(data.allowLoanedBooks, function (book, index, objs) {
            if (book) {
              $scope.allowLoanedBooks.push(book);
            }
          });
          $scope.totalBooks = data.totalBooks;
          $scope.loanedStatus = data.loanedStatus;
        }
      }
      function onError() {
        ngCordovaService.showToast("加载失败，请稍后再试");
      }
      function onFinally() {
        $ionicLoading.hide();
      }

      // ----------------------------------------------------------------------------------------------------------------

      //加入选书篮
      $scope.addBook = function () {
        uiService.ionicLoadingShow({
          templateTitle: "正在加入选书篮，请稍候...",
        });
        apiService.addBookCart($scope.bookDetail, {
          onSuccess: function (data, status, config, headers) {
            ngCordovaService.showToast("加入成功,请到“我的”中查看选书篮");
            $scope.loanedStatus = "已加入选书篮";
          },
          onError: function (data, status, config, headers) {
            ngCordovaService.showToast("加入失败，请稍后重试...");
            //
          },
          onFinally: function () {
            $ionicLoading.hide();
          },
        });
      };
      //到馆借阅提示
      $scope.toLibrary = function () {
        $ionicPopup.alert({
          title: "本册图书暂不支持线上借阅，请到馆借阅",
          okText: "确定",
        });
      };
      //预约
      $scope.goReserve = function () {
        uiService.ionicLoadingShow({ templateTitle: "正在预约，请稍候..." });
        apiService.reserveBook($scope.needBook, {
          onSuccess: function (data, status, config, headers) {
            console.log(data);
            if (data.status == "OK") {
              ngCordovaService.showToast("预约成功,请到“我的”中查看预约图书");
              $state.go("tab.mine");
            } else {
              ngCordovaService.showToast(data.data);
            }
          },
          onError: function (data, status, config, headers) {
            ngCordovaService.showToast("预约失败，请稍后重试...");
            //
          },
          onFinally: function () {
            $ionicLoading.hide();
          },
        });
      };

      // ----------------------------------------------------------------------------------------------------------------
      //
      $scope.retrieveBookDetail();

      // ----------------------------------------------------------------------------------------------------------------
      // 获取base controller，并将其扩展至当前controller中。
      var baseBookDetailCtrl = $controller("baseBookDetailCtrl", {
        $scope: $scope,
      });
      angular.extend(this, baseBookDetailCtrl);
      if ($scope.book) {
        $scope.init($scope.book.isbn); // init函数是base controller中的
      }

      // ----------------------------------------------------------------------------------------------------------------
      var onBookBorrowedListener = function (data) {
        // $scope.retrieveBookDetail();
        // 由于多处位置都会跳到借书的图书详情。临时不做监听处理，只打印一个log即可。
        console.log(
          "借书的图书详情页面，监听到还书事件。如下：\n" +
            angular.toJson(data, true)
        );
        // 借书成功之后，从当前可借图书中移除，替代自动查下速度快。
        var barCode = data.bar_code;
        if ($scope.allowLoanedBooks && barCode) {
          for (var i = 0; i < $scope.allowLoanedBooks.length; i++) {
            var allowLoanedBook = $scope.allowLoanedBooks[i];
            if (allowLoanedBook && barCode == allowLoanedBook.barCode) {
              $scope.allowLoanedBooks.splice(i, 1);
              $scope.loanedBook = allowLoanedBook; // 简单记录当前已经被借的图书
              break;
            }
          }
        }
      };
      // 监听和反监听
      broadcastService.register(
        broadcastConstants.names.onBookBorrowed,
        onBookBorrowedListener
      );
      $scope.$on("$destroy", function () {
        broadcastService.unregister(
          broadcastConstants.names.onBookBorrowed,
          onBookBorrowedListener
        );
      });

      // ----------------------------------------------------------------------------------------------------------------
      $ionicTabsDelegate.showBar(false);
    }
  );
